import { format as d3Format } from 'd3-format';
import { ESupportedLocales } from '@cloudflare/intl-core';
import moment from 'moment';
var DEFAULT_LOCALE = ESupportedLocales.en_US;
var DEFAULT_CURRENCY = 'USD';
var SI_PATTERN = /^([^a-zA-Z]*)([a-zA-Z]?$)/;
/**
 * Moment formatters per-locale examples.
 *
 *   en-US
 * -------------------
 * LongDate:            September 4, 1986
 * ShortDate:           Sep 4, 1986
 * Time:                1:30 PM
 * DateTime:            September 4, 1986
 * ISO8601:             1986-09-04T13:30:00-07:00
 * LongDayMonth:        4 September
 * ShortDayMonth:       4 Sep
 *
 *   de-DE
 * -------------------
 * LongDate:            4. September 1986
 * ShortDate:           4. Sep. 1986
 * Time:                13:30
 * DateTime:            4. September 1986 13:30
 * ISO8601:             1986-09-04T13:30:00-07:00
 * LongDayMonth:        4 September
 * ShortDayMonth:       4 Sep.
 */

export var DateFormatters;

(function (DateFormatters) {
  DateFormatters["LongDate"] = "LL";
  DateFormatters["ShortDate"] = "ll";
  DateFormatters["Time"] = "LT";
  DateFormatters["DateTime"] = "LLL";
  DateFormatters["ISO8601"] = "YYYY-MM-DDTHH:mm:ssZ";
  DateFormatters["LongDayMonth"] = "D MMMM";
  DateFormatters["ShortDayMonth"] = "D MMM";
  DateFormatters["DateTimeWithSeparators"] = "L LT";
  DateFormatters["DateTimeSecondsWithSeparators"] = "L LTS";
})(DateFormatters || (DateFormatters = {}));

export var formatDate = function formatDate(value, formatter) {
  var utc = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
  // TODO: Implement global UTC pyarreference fetching and formatting
  var formattedDate = moment(value);

  if (utc) {
    formattedDate = formattedDate.utc();
  }

  return formattedDate.format(formatter);
};
export var localizeNumberWithPrecision = function localizeNumberWithPrecision(number) {
  var locale = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : DEFAULT_LOCALE;
  return number.toLocaleString(locale, {
    maximumFractionDigits: 20
  });
};
export var formatNumber = function formatNumber(number) {
  var locale = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : DEFAULT_LOCALE;
  var useSI = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
  var decimalPlaces = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 2;
  var trimInsignificantZeros = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : true;
  var val = number;
  var suffix = ''; // SI units = International system of units.
  // Don't use SI units for sub-1 numbers (milli, micro, etc)

  if (useSI && number >= 1) {
    // d3Format notes:
    // 1.) s refers to decimal notation with an SI prefix, rounded to significant digits.
    // Example input and output: d3.format("s")(1500);  // "1.50000k".
    // 2.) The exec() method on a regular expression executes a search for a match in a
    // specified string. Returns a result array, or null.
    var siMatch = SI_PATTERN.exec(d3Format('s')(number)); // Split value from SI unit.
    // siMatch is an array that looks like:
    // ["479.247M", "479.247", "M", index: 0, input: "479.247M", groups: undefined]

    if (siMatch) {
      val = +siMatch[1];
      suffix = siMatch[2] || '';
    }
  } // d3Format notes:
  // 1.) ~ trims insignificant trailing digits.
  // 2.) f refers to fixed point notation.


  var formatted = d3Format(".".concat(decimalPlaces).concat(trimInsignificantZeros ? '~' : '', "f"))(val);
  return Number(formatted).toLocaleString(locale) + suffix;
};
export var formatCurrency = function formatCurrency() {
  var value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {
    locale: DEFAULT_LOCALE
  };
  return Number(value).toLocaleString(options.locale, {
    style: 'currency',
    // We should *always* display the currency that was billed in (USD).
    currency: DEFAULT_CURRENCY,
    // In the future—we can make these properties configurable.
    minimumFractionDigits: 2,
    maximumFractionDigits: 2
  })
  /**
   * `Number.toLocaleString` with `style: 'currency'` behaves differently
   * across vendors/browsers. On top of this, the spec/dcoumentation is
   * mysterious on how it _should_ work. Read more here—
   * https://stackoverflow.com/a/56161892.
   *
   * tl;dr
   * -------------------
   * When formatting in `USD` currency with a non-`en-US` locale,
   * `toLocaleString` will format the string like so: `400,00 $`. Looks
   * fine, but the weird part is that the space is actually a unicode
   * 160 char (non-breaking space), not a regular unicode 30 char (normal
   * space). Furthermore, our Trans/DOMPurify implementation will try to
   * sanitize this string and renders `400,00&nbsp;$` to the DOM.
   *
   * So.. to fix this, we simply replace any abnormal "space" unicode chars
   * with a regular space. The regex comes from—
   * https://stackoverflow.com/a/51399843.
   */
  .replace(/[\u202F\u00A0\u2000\u2001\u2003]/, ' ');
};
/**
 * @param {Number} number
 * @param {String} locale
 * @param {Boolean} [includeDecimals]
 * @return {String}
 */

export var formatBytes = function formatBytes(number) {
  var locale = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : DEFAULT_LOCALE;
  var includeDecimals = arguments.length > 2 ? arguments[2] : undefined;
  return formatNumber(number, locale, true, includeDecimals ? 2 : 0) // Add space between value and SI unit
  .replace(/([a-zA-Z]?)$/, ' $1') + 'B';
};
/**
 * @param {Number} number
 * @param {String} locale
 * @param {Boolean} [useSI]
 * @param {Number} [decimalPlaces]
 * @param {Boolean} [trimInsignificantZeros]
 * @return {String}
 */

export var formatNumberForAnalytics = function formatNumberForAnalytics(number) {
  var locale = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : DEFAULT_LOCALE;
  var useSI = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
  var decimalPlaces = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 2;
  var trimInsignificantZeros = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : true;
  return formatNumber(number, locale, useSI, decimalPlaces, trimInsignificantZeros);
};